@@ -134,10 +134,10 @@ int pgnpl(int nmax, int n) {
   }
   /* Now that we have a copy of the strings we can easily upcase them -
      makes parsing easier */
-  for(char* p = xopt_cp; *p; p++ )
-      *p = toupper(*p);
-  for(char* p = yopt_cp; *p; p++ )
-      *p = toupper(*p);
+  for(char* px = xopt_cp; *px; px++ )
+      *px = toupper(*px);
+  for(char* py = yopt_cp; *py; py++ )
+      *py = toupper(*py);
 
   /* X-axis */
   if( strchr(xopt_cp, 'Z') ) {
@@ -1082,9 +1082,9 @@ void pgtbx7(char const* suptyp, char signf, char asign, int ival[3], double rval
     if( writ[0] ) {
         /* depending on wether to print the sign or not choose the correct
          * format */
-        char const* fmt = (signf=='D' && asign!=' ') ? "%1$c%2$d%3$s" : "%2$d%3$s" ;
+        char const* fmt = (signf=='D' && asign!=' ') ? "%3$c%1$d%2$s" : "%1$d%2$s" ;
         *last  = *tlen;
-        nch   = sprintf(&text[*tlen], fmt, asign, ival[0], suppnt[0]);
+        nch   = sprintf(&text[*tlen], fmt, ival[0], suppnt[0], asign);
         *tlen = *tlen + nch;
     }
 
